@@ -1356,13 +1356,14 @@ usage(void)
     "    -l file                  Script source file\n",
     "    -s file                  Script source file\n",
     "    -p alias=path            Define file search path 'alias'\n",
-    "    -D name=value		  Set a Prolog flag\n",
+    "    -D name=value            Set a Prolog flag\n",
     "    -O                       Optimised compilation\n",
     "    --on-error=style         One of print, halt or status\n",
     "    --on-warning=style       One of print, halt or status\n",
     "    --tty[=bool]             (Dis)allow tty control\n",
     "    --packs[=bool]           Do (not) attach add-ons\n",
     "    --signals[=bool]         Do (not) modify signal handling\n",
+    "    --sigalert[=num]         Use signal num for alerting threads\n",
     "    --threads[=bool]         Do (not) allow for threads\n",
     "    --debug[=bool]           Do (not) generate debug info\n",
     "    --debug-on-interrupt[=bool] Trap the debugger on interrupt\n",
@@ -1767,52 +1768,80 @@ warning(const char *fm, ...)
 #define ctime_r(timep, buf) strcpy(buf, ctime(timep))
 #endif
 
-static bool
-vsysError(const char *fm, va_list args)
+void
+printCrashContext(const char *btname)
 { GET_LD
-  static int active = 0;
   time_t now;
   char tbuf[48];
+  int btflags = 0;
+  static bool running = FALSE;
 
-  if ( active++ )
-  { abort();
-  }
+  if ( running )
+    Sdprintf("Recursive crash; omitting crash report\n");
+  running = TRUE;
 
   now = time(NULL);
   ctime_r(&now, tbuf);
   tbuf[24] = '\0';
+  Sdprintf("Time: %s\n", tbuf);
+
+  if ( LD )
+  { Sdprintf("Inferences: %" PRIu64 "\n", LD->statistics.inferences);
+    if ( gc_status.active )
+    { Sdprintf("Running GC: %" PRIu64 "-th garbage collection]\n",
+	       LD->gc.stats.totals.collections);
+      unblockSignals(&LD->gc.saved_sigmask);
+      btflags |= PL_BT_SAFE;
+    }
+  }
 
 #ifdef O_PLMT
-{ int tid = PL_thread_self();
+  int tid = PL_thread_self();
   atom_t alias;
   const pl_wchar_t *name = L"";
 
   if ( PL_get_thread_alias(tid, &alias) )
     name = PL_atom_wchars(alias, NULL);
 
-  SfprintfX(Serror, "[PROLOG SYSTEM ERROR:  Thread %d (%Ws) at %s\n\t",
-	    tid, name, tbuf);
-}
-#else
-  Sfprintf(Serror, "[PROLOG SYSTEM ERROR: at %s\n\t", tbuf);
+  SdprintfX("Thread: %d (%Ws)\n", tid, name);
 #endif
-  Svfprintf(Serror, fm, args);
-  if ( gc_status.active )
-  { Sfprintf(Serror,
-	    "\n[While in %ld-th garbage collection]\n",
-	    LD->gc.stats.totals.collections);
-    unblockSignals(&LD->gc.saved_sigmask);
+
+  print_backtrace_named(btname);
+  if ( LD )
+  { if ( getenv("SWIPL_DEBUG_GC_STACK") )
+    { if ( LD->shift_status.inferences )
+      { Sdprintf("Last stack shift at %" PRIu64 " inferences\n",
+		 LD->shift_status.inferences);
+	print_backtrace_named("SHIFT");
+      }
+      if ( LD->gc.inferences )
+      { Sdprintf("Last garbage collect at %" PRIu64 " inferences\n",
+		 LD->gc.inferences);
+	print_backtrace_named("GC");
+      }
+    }
+    Sdprintf("\n\nPROLOG STACK:\n");
+    PL_backtrace(10, btflags);
   }
 
-  Sfprintf(Serror, "\n");
-  save_backtrace("SYSERROR");
-  print_backtrace_named("SYSERROR");
+  running = FALSE;
+}
 
-#if defined(O_DEBUGGER)
-  Sfprintf(Serror, "\n\nPROLOG STACK:\n");
-  PL_backtrace(10, 0);
-  Sfprintf(Serror, "]\n");
-#endif /*O_DEBUGGER*/
+
+
+static bool
+vsysError(const char *fm, va_list args)
+{ static int active = 0;
+
+  if ( active++ )
+    abort();
+
+  Sfprintf(Serror, "\nERROR: System error: ");
+  Svfprintf(Serror, fm, args);
+  Sfprintf(Serror, "\n");
+
+  save_backtrace("SYSERROR");
+  printCrashContext("SYSERROR");
 
   if ( !(true(Sinput, SIO_ISATTY) &&
 	 true(Serror, SIO_ISATTY)) ||
